@@ -1,10 +1,22 @@
 // AlphIO.h
 //
-/////////////////////////////////////////////////////////////////////////////
+// Copyright (c) 2007 The Dasher Team
 //
-// Copyright (c) 2002 Iain Murray
+// This file is part of Dasher.
 //
-/////////////////////////////////////////////////////////////////////////////
+// Dasher is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+//
+// Dasher is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Dasher; if not, write to the Free Software 
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifndef __AlphIO_h__
 #define __AlphIO_h__
@@ -13,11 +25,11 @@
 //#include "Alphabet.h"
 #include "GroupInfo.h"
 
-#ifdef _WIN32
-#include "../../Common/Expat/lib/expat.h"
-#else
+//#ifdef _WIN32
+//#include "../../Common/Expat/lib/expat.h"
+//#else
 #include <expat.h>
-#endif
+//#endif
 
 #include <string>
 #include <map>
